from __future__ import unicode_literals

from django.apps import apps
from django.contrib.sites.models import SITE_CACHE
from django.db.models import Q
from django.core.exceptions import ValidationError, ImproperlyConfigured
from django.http.request import split_domain_port
from django.conf import settings
from django.utils.lru_cache import lru_cache

from allowedsites import Sites as ASites, CachedAllowedSites as ACachedAllowedSites

from .exceptions import NoRequestFound


def load_settings(app_settings):
    for app_setting in dir(app_settings):
        ## only import settings, not imported modules or methods
        set_settings = dir(settings)
        if app_setting == app_setting.upper() and app_setting not in set_settings:
            setattr(settings, app_setting, getattr(app_settings, app_setting))

## Model Validation


def domain_available(obj, site_klass):
    from django.utils.translation import ugettext_lazy as _

    qs = site_klass.objects.all()
    q2 = Q(aliases__domain__iexact=obj.domain)
    if obj.pk and obj.pk is not None:
      if isinstance(obj, site_klass):
        qs = qs.exclude(pk=obj.pk)
      else:
        q2 = ~Q(aliases__pk=obj.pk) & q2
    try:
        qs.get(Q(domain__iexact=obj.domain) | q2)
        raise ValidationError(_("A site or an alias with that domain name already exists"),
                              code='invalid')
    except site_klass.DoesNotExist:
        pass


## AppConfig

def _get_host(request=None):
    if request is None:
        if 'airavata.middleware.ThreadLocalMiddleware' in settings.MIDDLEWARE_CLASSES:
            from threadlocals.threadlocals import get_thread_variable
            host = get_thread_variable('requested_host')
            if host is None:
                raise NoRequestFound("HostName could not be retrieved")
            return host
        else:
            raise ImproperlyConfigured("You should either provide a request or install threadlocals")
    domain_host, domain_port = split_domain_port(request.get_host())
    return domain_host


def _get_site_by_request(self, request=None):
    host = _get_host(request)
    # Looking for domain in django.contib.site.Site and airavata.SiteAlias
    if host not in SITE_CACHE:
      site = self.get(Q(domain__iexact=host) | Q(aliases__domain__iexact=host))
      SITE_CACHE[host] = site
    return SITE_CACHE[host]


def get_current_site(request=None):
    if apps.is_installed('django.contrib.sites'):
        from .models import Site
        if not hasattr(settings, 'SITE_ID') and request is None:
            return _get_site_by_request(Site.objects, request)
        else:
            return Site.objects.get_current(request)
    else:
        from django.contrib.sites.requests import RequestSite
        return RequestSite(request)


## Loaders

def get_domain_path(domain):
    if settings.POLLA_REPLACE_DOTS_IN_DOMAINS:
        domain = domain.replace('.', '_')
    return domain.lower()


def get_current_path(request=None):
    try:
        site = get_current_site(request)
        return get_domain_path(site.domain)
    except NoRequestFound:
        ## Request hasn't been loaded, rather than crashing we will return a blank value
        return ''


## Allowed hosts - adapted from https://github.com/kezabelle/django-allowedsites


class Sites(ASites):

    def get_raw_aliases(self):
        from .models import SiteAlias
        return SiteAlias.objects.all().iterator()

    def get_domains(self):
        """
        Yields domains *without* any ports defined, as that's what
        `validate_host` wants
        """
        raw_sites = self.get_raw_sites()
        raw_aliases = self.get_raw_aliases()
        domains = set()
        for domain_list in (raw_sites, raw_aliases):
            raw_domains = (item.domain for item in domain_list)
            for domain in raw_domains:
                domain_host, domain_port = split_domain_port(domain)
                domains.add(domain_host)

        return frozenset(domains)


class AllowedSites(Sites):

    __slots__ = ('defaults',)


class CachedAllowedSites(Sites, ACachedAllowedSites):

    __slots__ = ('defaults', 'key')


## Cache invalidation

def register_signals(model):
    from django.db.models.signals import post_save, post_delete
    post_save.connect(CachedAllowedSites.update_cache, sender=model, dispatch_uid='update_allowedsites')
    post_delete.connect(CachedAllowedSites.update_cache, sender=model, dispatch_uid='update_allowedsites')


## Resolver

@lru_cache(maxsize=None)
def get_resolver_for_site(urlconf, site):
    from django.core.urlresolvers import RegexURLResolver
    if urlconf is None:
        urlconf = settings.ROOT_URLCONF
    return RegexURLResolver(r'^/', urlconf)


def get_resolver(urlconf):
    current_site = get_current_path()
    return get_resolver_for_site(urlconf, current_site)


@lru_cache(maxsize=None)
def get_ns_resolver_for_site(ns_pattern, resolver, site):
    from django.core.urlresolvers import RegexURLResolver
    ns_resolver = RegexURLResolver(ns_pattern, resolver.url_patterns)
    return RegexURLResolver(r'^/', [ns_resolver])


def get_ns_resolver(ns_pattern, resolver):
    current_site = get_current_path()
    return get_ns_resolver_for_site(ns_pattern, resolver, current_site)


## Upload path builder

def upload_path(path_or_callable=''):
    if callable(path_or_callable):
        return lambda instance, filename: '{}/{}'.format(get_current_path(), path_or_callable(instance, filename))
    else:
        return lambda instance, filename: '{}/{}{}'.format(
            get_current_path(),
            '{}/'.format(path_or_callable) if path_or_callable != '' else '',
            filename
        )



import io

from django.conf import settings
from django.core.exceptions import SuspiciousFileOperation
from django.template.base import TemplateDoesNotExist
from django.template.loaders.base import Loader as BaseLoader
from django.utils._os import safe_join
from .utils import get_current_path
from .exceptions import NoRequestFound


class Loader(BaseLoader):
    is_usable = True

    def get_template_sources(self, template_name, template_dirs=None):
        try:
            current = get_current_path()
        except NoRequestFound:
            return False

        try:
            return safe_join(settings.POLLA_SITES_DIR, current, 'templates', template_name)
        except SuspiciousFileOperation:
            return False

    def load_template_source(self, template_name, template_dirs=None):
        tried = ''
        filepath = self.get_template_sources(template_name, template_dirs)
        if filepath:
            try:
                with io.open(filepath, encoding=self.engine.file_charset) as fp:
                    return fp.read(), filepath
            except IOError:
                tried = filepath
        error_msg = "Tried %s" % tried
        raise TemplateDoesNotExist(error_msg)

    load_template_source.is_usable = True



import os
from collections import OrderedDict

from django.contrib.staticfiles.finders import FileSystemFinder, BaseFinder
from django.core.files.storage import FileSystemStorage
from django.contrib.sites.models import Site
from django.conf import settings

from airavata.utils import get_domain_path


class SiteFinder(FileSystemFinder):

    def __init__(self, app_name=None, *args, **kwargs):
        self.locations = []
        self.storages = OrderedDict()
        for site in Site.objects.all():
            current = get_domain_path(site.domain)
            root = os.path.join(settings.POLLA_SITES_DIR, current, 'static')
            if os.path.exists(root) and (current, root) not in self.locations:
                self.locations.append((current, root))

        for prefix, root in self.locations:
            filesystem_storage = FileSystemStorage(location=root)
            filesystem_storage.prefix = prefix
            self.storages[root] = filesystem_storage
        BaseFinder.__init__(self, *args, **kwargs)



from django.apps import AppConfig


class PollaAppConfig(AppConfig):

    name = 'airavata'
    verbose_name = 'Polla'

    def ready(self):
        from .utils import _get_site_by_request, domain_available, load_settings
        from . import settings as airavata_settings
        from django.contrib.sites.models import SiteManager, Site

        load_settings(airavata_settings)

        from django.conf import settings
        if 'airavata.middleware.ThreadLocalMiddleware' in settings.MIDDLEWARE_CLASSES:
            from .utils import get_current_site
            from django.contrib.sites import shortcuts
            shortcuts.get_current_site = get_current_site

        SiteManager._get_site_by_request = _get_site_by_request

        old_site_clean = getattr(Site, 'clean', None)

        def site_clean(self):
            domain_available(self, Site)
            if old_site_clean is not None:
                old_site_clean(self)

        Site.clean = site_clean



from __future__ import unicode_literals
import copy
import importlib
from django.core.exceptions import ImproperlyConfigured
from .utils import get_current_path
from .exceptions import NoRequestFound
from django.conf import settings


POLLA_PATCHED_RESOLVER = False

class UrlPatterns(object):

    def __init__(self, defaults=[]):
        if not settings.POLLA_REPLACE_DOTS_IN_DOMAINS:
            raise ImproperlyConfigured("POLLA_REPLACE_DOTS_IN_DOMAINS needs to be set to True in order to use this functionality")
        self.defaults = defaults
        self.extras = []

    def get_site_urls(self):
        try:
            current = get_current_path()
        except NoRequestFound:
            ## No request was found, falling back to defaults
            return None
        try:
            site_urls_module = importlib.import_module('sites.{}.urls'.format(current))
            patterns = getattr(site_urls_module, 'urlpatterns', None)
            return patterns
        except ImportError:
            pass
        except ValueError:
            pass
        return None

    def get_non_extra_urls(self):
        site_urls = self.get_site_urls()
        if site_urls is None:
            site_urls = copy.copy(self.defaults)
        return site_urls

    def get_current_urls(self):
        extras = copy.copy(self.extras)
        site_urls = self.get_non_extra_urls()
        site_urls.extend(extras)
        return site_urls

    def __iter__(self):
        return iter(self.get_current_urls())

    def __repr__(self):
        return '<{mod}.{cls}>'.format(
            mod=self.__class__.__module__,
            cls=self.__class__.__name__
        )

    def __str__(self):
        return '{}'.format(self.get_current_urls())

    __unicode__ = __str__

    def __contains__(self, other):
        if other in self.extras:
            return True
        urls = self.get_non_extra_urls()
        return other in urls

    def __len__(self):
        urls = self.get_current_urls()
        return len(urls)

    def __nonzero__(self):
        if len(self.extras):
            return True
        urls = self.get_non_extra_urls()
        return len(urls) > 0

    __bool__ = __nonzero__

    def __eq__(self, other):
        if self.defaults != other.defaults or self.extras != other.extras:
            return False
        return True

    def __add__(self, other):
        self.extras.extend(other)
        return self

    def __sub__(self, other):
        raise NotImplemented

    def __reversed__(self):
        to_reverse = list(self.get_current_urls())
        return reversed(to_reverse)

if not POLLA_PATCHED_RESOLVER:
    from .utils import get_resolver, get_ns_resolver
    from django.core import urlresolvers
    urlresolvers.get_resolver = get_resolver
    urlresolvers.get_ns_resolver = get_ns_resolver
    POLLA_PATCHED_RESOLVER = True



from __future__ import unicode_literals
from django.utils.encoding import python_2_unicode_compatible

from django.db import models
from django.contrib.sites.models import Site, _simple_domain_name_validator
from django.utils.translation import ugettext_lazy as _

from .utils import domain_available


@python_2_unicode_compatible
class SiteAlias(models.Model):

    site = models.ForeignKey(Site, verbose_name=_('site'), related_name='aliases')
    domain = models.CharField(_('domain name alias'), max_length=100, unique=True,
                              validators=[ _simple_domain_name_validator])

    class Meta:
        verbose_name = "SiteAlias"
        verbose_name_plural = "SiteAliases"

    def __str__(self):
        return '{} ({})'.format(self.domain, self.site.domain)

    def clean(self):
        domain_available(self, Site)
        super(SiteAlias, self).clean()



from django.contrib.sites.shortcuts import get_current_site


class SiteFilteredViewMixin(object):

    site_field = 'site'

    def get_queryset(self):
        qs = super(SiteFilteredViewMixin, self).get_queryset()
        kwargs = {
          self.site_field: get_current_site(self.request),
        }
        return qs.filter(**kwargs)



import os

from django import template
from django.contrib.staticfiles.storage import staticfiles_storage
from django.templatetags.static import StaticNode
from django.conf import settings

from ..utils import get_current_path

register = template.Library()


def static(path):
    site_path = get_current_path()
    if os.path.exists(os.path.join(settings.POLLA_SITES_DIR, site_path, 'static', path)):
        return staticfiles_storage.url(os.path.join(site_path, path))
    return staticfiles_storage.url(path)


class StaticFilesNode(StaticNode):

    def url(self, context):
        path = self.path.resolve(context)
        return static(path)


@register.tag('static')
def do_static(parser, token):
    return StaticFilesNode.handle_token(parser, token)



import os


POLLA_BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

## Sites specific files base directory
POLLA_SITES_DIR = os.path.join(POLLA_BASE_DIR, 'sites')

## should dots from domain names be replaced by underscores for
## paths used in templates, staticfiles and urls loaders
POLLA_REPLACE_DOTS_IN_DOMAINS = False



from django.http.request import split_domain_port
from threadlocals.threadlocals import set_thread_variable


class ThreadLocalMiddleware(object):

    """Middleware that puts the request object in thread local storage."""

    def process_request(self, request):
        domain_host, domain_port = split_domain_port(request.get_host())
        set_thread_variable('requested_host', domain_host)



from django.contrib import admin

from .models import SiteAlias


@admin.register(SiteAlias)
class SiteAliasAdmin(admin.ModelAdmin):

    list_display = ('domain', 'get_site_domain', 'get_site_name')

    def get_site_name(self, obj):
        return obj.site.name
    get_site_name.short_description = 'Site name'
    get_site_name.admin_order_field = 'site__name'

    def get_site_domain(self, obj):
        return obj.site.name
    get_site_domain.short_description = 'Main domain'
    get_site_domain.admin_order_field = 'site__domain'



import os
from setuptools import setup


try:
    from pypandoc import convert
    README = convert('README.md', 'rst')
except ImportError:
    with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
        README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='airavata',
    version='0.9.3',
    packages=['airavata', 'airavata.management.commands', 'airavata.templatetags'],
    include_package_data=True,
    license='BSD License',
    description='Multiple dynamic sistes with Django',
    long_description=README,
    url='http://bb.levit.be/djang-airavata',
    author='Levit SCS',
    author_email='emma@levit.be',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    install_requires=['Django>=1.8', 'django-allowedsites']
)



from __future__ import unicode_literals

from django.core.management.base import BaseCommand, CommandError
from django.core.exceptions import ValidationError
from django.db.models import Q

from django.contrib.sites.models import Site
from ...models import SiteAlias


class Command(BaseCommand):
    help = 'Sets the primary domain name (and optionnally localhost alias)'

    def __init__(self, *args, **kwargs):
        super(Command, self).__init__(*args, **kwargs)
        self.domain_field = Site._meta.get_field('domain')

    def add_arguments(self, parser):
        parser.add_argument('domain_name', nargs='?', type=str, default='')
        parser.add_argument('--noinput', action='store_false', dest='interactive', default=True,
            help='Tells Django to NOT prompt the user for input of any kind.')
        parser.add_argument('--localhost-alias', dest='do_alias',
            default=None, action='store_true',
            help='Create localhost alias if localhost is not configured')

    def get_input_data(self, field, message, default=None):
        raw_value = input(message)
        if default and raw_value == '':
             raw_value = default
        try:
             val = field.clean(raw_value, None)
        except ValidationError as e:
             self.stderr.write("Error: %s" % '; '.join(e.messages))
             val = None

        return val

    def handle(self, **options):
        domain = options.get('domain_name', None)
        if options.get('interactive', True):
            while domain is None or domain == '':
                domain = self.get_input_data(self.domain_field, 'Enter the main domain name [localhost]: ', 'localhost')
        elif domain is None or domain == '':
            raise CommandError('Please supply a domain name when using --noinput')

        # Try to find the default example.com
        # Fallback to the first site created or None
        try:
            site = Site.objects.get(domain='example.com')
        except Site.DoesNotExist:
            try:
                site = Site.objects.order_by('id').first()
            except Site.DoesNotExist:
                site = None

        if site is None:
            site = Site.objects.create(domain=domain, name=domain)
        else:
            site.domain = domain
            site.save()

        if domain != 'localhost':
            found = Site.objects.filter(Q(domain__iexact='localhost') |
                Q(aliases__domain__iexact='localhost')).count()
            if found == 0:
                do_alias = options.get('do_alias', None)
                if do_alias is None and options.get('interactive', True):
                    while do_alias is None:
                        do_create = input('No site found for loclhost would you like to create and alias? [y/N] ')
                        if do_create.lower() in 'yn':
                            do_alias = True if do_create.lower() else False
                if do_alias is not None and do_alias:
                    SiteAlias.objects.create(site=site, domain='localhost')

        self.stdout.write('Successfully updated site "{}"'.format(site))
